@@ -20,15 +20,20 @@
 
 #include <QDebug>
 #include <QDir>
-#include <QtCore/QProcess>
 #include <kconfiggroup.h>
 #include <kdesktopfile.h>
 #include <kmimetypetrader.h>
 #include <kservicefactory_p.h>
+#include <qtemporarydir.h>
 #include <qtemporaryfile.h>
 #include <qtest.h>
-#include "kmimeassociations.h"
+#include "kmimeassociations_p.h"
+#include <kbuildsycoca_p.h>
 #include <ksycoca.h>
+#include "ksycoca_p.h"
+#include <QMimeDatabase>
+#include <QMimeType>
+#include <QSignalSpy>
 
 // We need a factory that returns the same KService::Ptr every time it's asked for a given service.
 // Otherwise the changes to the service's serviceTypes by KMimeAssociationsTest have no effect
@@ -112,8 +117,6 @@ private Q_SLOTS:
         QStandardPaths::enableTestMode(true);
         qputenv("XDG_CURRENT_DESKTOP", "KDE");
 
-        extern KSERVICE_EXPORT bool kservice_require_kded;
-        kservice_require_kded = false;
 
         m_localConfig = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + QLatin1Char('/');
         QDir(m_localConfig).removeRecursively();
@@ -126,15 +129,15 @@ private Q_SLOTS:
 
         // Create fake application (associated with text/plain in mimeapps.list)
         fakeTextApplication = m_localApps + "faketextapplication.desktop";
-        writeAppDesktopFile(fakeTextApplication, QStringList() << "text/plain");
+        writeAppDesktopFile(fakeTextApplication, QStringList() << QStringLiteral("text/plain"));
 
         // Create fake application (associated with text/plain in mimeapps.list)
         fakeTextApplicationPrefixed = m_localApps + "fakepfx/faketextapplicationpfx.desktop";
-        writeAppDesktopFile(fakeTextApplicationPrefixed, QStringList() << "text/plain");
+        writeAppDesktopFile(fakeTextApplicationPrefixed, QStringList() << QStringLiteral("text/plain"));
 
         // A fake "default" application for text/plain (high initial preference, but not in mimeapps.list)
         fakeDefaultTextApplication = m_localApps + "fakedefaulttextapplication.desktop";
-        writeAppDesktopFile(fakeDefaultTextApplication, QStringList() << "text/plain", 9);
+        writeAppDesktopFile(fakeDefaultTextApplication, QStringList() << QStringLiteral("text/plain"), 9);
 
         // An app (like emacs) listing explicitly the derived mimetype (c-src); not in mimeapps.list
         // This interacted badly with mimeapps.list listing another app for text/plain, but the
@@ -143,28 +146,29 @@ private Q_SLOTS:
         //
         // Also include aliases (msword), to check they don't cancel each other out.
         fakeCSrcApplication = m_localApps + "fakecsrcmswordapplication.desktop";
-        writeAppDesktopFile(fakeCSrcApplication, QStringList() << "text/plain" << "text/c-src" << "application/vnd.ms-word" << "application/msword", 8);
+        writeAppDesktopFile(fakeCSrcApplication, QStringList() << QStringLiteral("text/plain") << QStringLiteral("text/c-src") << QStringLiteral("application/vnd.ms-word") << QStringLiteral("application/msword"), 8);
 
         fakeJpegApplication = m_localApps + "fakejpegapplication.desktop";
-        writeAppDesktopFile(fakeJpegApplication, QStringList() << "image/jpeg");
+        writeAppDesktopFile(fakeJpegApplication, QStringList() << QStringLiteral("image/jpeg"));
 
         fakeArkApplication = m_localApps + "fakearkapplication.desktop";
-        writeAppDesktopFile(fakeArkApplication, QStringList() << "application/zip");
+        writeAppDesktopFile(fakeArkApplication, QStringList() << QStringLiteral("application/zip"));
 
         fakeHtmlApplication = m_localApps + "fakehtmlapplication.desktop";
-        writeAppDesktopFile(fakeHtmlApplication, QStringList() << "text/html");
+        writeAppDesktopFile(fakeHtmlApplication, QStringList() << QStringLiteral("text/html"));
 
         fakeHtmlApplicationPrefixed = m_localApps + "fakepfx/fakehtmlapplicationpfx.desktop";
-        writeAppDesktopFile(fakeHtmlApplicationPrefixed, QStringList() << "text/html");
+        writeAppDesktopFile(fakeHtmlApplicationPrefixed, QStringList() << QStringLiteral("text/html"));
 
-        // Update ksycoca in ~/.kde-unit-test after creating the above
+        // Update ksycoca in ~/.qttest after creating the above
         runKBuildSycoca();
 
         // Create factory on the heap and don't delete it. This must happen after
         // Sycoca is built, in case it did not exist before.
         // It registers to KSycoca, which deletes it at end of program execution.
         KServiceFactory *factory = new FakeServiceFactory(KSycoca::self());
-        QCOMPARE(KServiceFactory::self(), factory); // ctor sets s_self
+        KSycocaPrivate::self()->m_serviceFactory = factory;
+        QCOMPARE(KSycocaPrivate::self()->serviceFactory(), factory);
 
         // For debugging: print all services and their storageId
 #if 0
@@ -175,7 +179,7 @@ private Q_SLOTS:
         }
 #endif
 
-        KService::Ptr fakeApplicationService = KService::serviceByStorageId("faketextapplication.desktop");
+        KService::Ptr fakeApplicationService = KService::serviceByStorageId(QStringLiteral("faketextapplication.desktop"));
         QVERIFY(fakeApplicationService);
 
         m_mimeAppsFileContents = "[Added Associations]\n"
@@ -184,25 +188,28 @@ private Q_SLOTS:
                                  "text/plain=faketextapplication.desktop;fakepfx-faketextapplicationpfx.desktop;gvim.desktop;wine.desktop;idontexist.desktop;\n"
                                  // test alias resolution
                                  "application/x-pdf=fakejpegapplication.desktop;\n"
+                                 // test x-scheme-handler (#358159) (missing trailing ';' as per xdg-mime bug...)
+                                 "x-scheme-handler/mailto=faketextapplication.desktop\n"
                                  "[Added KParts/ReadOnlyPart Associations]\n"
                                  "text/plain=katepart.desktop;\n"
                                  "[Removed Associations]\n"
                                  "image/jpeg=firefox.desktop;\n"
                                  "text/html=gvim.desktop;abiword.desktop;\n";
         // Expected results
-        preferredApps["image/jpeg"] << "fakejpegapplication.desktop";
-        preferredApps["application/pdf"] << "fakejpegapplication.desktop";
-        preferredApps["text/plain"] << "faketextapplication.desktop"
-                                    << "fakepfx-faketextapplicationpfx.desktop"
-                                    << "gvim.desktop";
-        preferredApps["text/x-csrc"] << "faketextapplication.desktop"
-                                     << "fakepfx-faketextapplicationpfx.desktop"
-                                     << "gvim.desktop";
-        preferredApps["text/html"] << "fakehtmlapplication.desktop"
-                                   << "fakepfx-fakehtmlapplicationpfx.desktop";
-        preferredApps["application/msword"] << "fakecsrcmswordapplication.desktop";
-        removedApps["image/jpeg"] << "firefox.desktop";
-        removedApps["text/html"] << "gvim.desktop" << "abiword.desktop";
+        preferredApps[QStringLiteral("image/jpeg")] << QStringLiteral("fakejpegapplication.desktop");
+        preferredApps[QStringLiteral("application/pdf")] << QStringLiteral("fakejpegapplication.desktop");
+        preferredApps[QStringLiteral("text/plain")] << QStringLiteral("faketextapplication.desktop")
+                                    << QStringLiteral("fakepfx-faketextapplicationpfx.desktop")
+                                    << QStringLiteral("gvim.desktop");
+        preferredApps[QStringLiteral("text/x-csrc")] << QStringLiteral("faketextapplication.desktop")
+                                     << QStringLiteral("fakepfx-faketextapplicationpfx.desktop")
+                                     << QStringLiteral("gvim.desktop");
+        preferredApps[QStringLiteral("text/html")] << QStringLiteral("fakehtmlapplication.desktop")
+                                   << QStringLiteral("fakepfx-fakehtmlapplicationpfx.desktop");
+        preferredApps[QStringLiteral("application/msword")] << QStringLiteral("fakecsrcmswordapplication.desktop");
+        preferredApps[QStringLiteral("x-scheme-handler/mailto")] << QStringLiteral("faketextapplication.desktop");
+        removedApps[QStringLiteral("image/jpeg")] << QStringLiteral("firefox.desktop");
+        removedApps[QStringLiteral("text/html")] << QStringLiteral("gvim.desktop") << QStringLiteral("abiword.desktop");
 
         // Clean-up non-existing apps
         removeNonExisting(preferredApps);
@@ -218,10 +225,13 @@ private Q_SLOTS:
     void testParseSingleFile()
     {
         KOfferHash offerHash;
-        KMimeAssociations parser(offerHash, KServiceFactory::self());
+        KMimeAssociations parser(offerHash, KSycocaPrivate::self()->serviceFactory());
 
-        QTemporaryFile tempFile;
-        QVERIFY(tempFile.open());
+        QTemporaryDir tempDir;
+        QVERIFY(tempDir.isValid());
+
+        QFile tempFile(tempDir.path() + "/mimeapps.list");
+        QVERIFY(tempFile.open(QIODevice::WriteOnly));
         tempFile.write(m_mimeAppsFileContents);
         const QString fileName = tempFile.fileName();
         tempFile.close();
@@ -266,11 +276,14 @@ private Q_SLOTS:
     void testGlobalAndLocalFiles()
     {
         KOfferHash offerHash;
-        KMimeAssociations parser(offerHash, KServiceFactory::self());
+        KMimeAssociations parser(offerHash, KSycocaPrivate::self()->serviceFactory());
 
         // Write global file
-        QTemporaryFile tempFileGlobal;
-        QVERIFY(tempFileGlobal.open());
+        QTemporaryDir tempDirGlobal;
+        QVERIFY(tempDirGlobal.isValid());
+
+        QFile tempFileGlobal(tempDirGlobal.path() + "/mimeapps.list");
+        QVERIFY(tempFileGlobal.open(QIODevice::WriteOnly));
         QByteArray globalAppsFileContents = "[Added Associations]\n"
                                             "image/jpeg=firefox.desktop;\n" // removed by local config
                                             "text/html=firefox.desktop;\n" // mdv
@@ -280,8 +293,11 @@ private Q_SLOTS:
         tempFileGlobal.close();
 
         // We didn't keep it, so we need to write the local file again
-        QTemporaryFile tempFile;
-        QVERIFY(tempFile.open());
+        QTemporaryDir tempDir;
+        QVERIFY(tempDir.isValid());
+
+        QFile tempFile(tempDir.path() + "/mimeapps.list");
+        QVERIFY(tempFile.open(QIODevice::WriteOnly));
         tempFile.write(m_mimeAppsFileContents);
         const QString fileName = tempFile.fileName();
         tempFile.close();
@@ -289,23 +305,23 @@ private Q_SLOTS:
         parser.parseMimeAppsList(globalFileName, 1000);
         parser.parseMimeAppsList(fileName, 1050); // += 50 is correct.
 
-        QList<KServiceOffer> offers = offerHash.offersFor("image/jpeg");
+        QList<KServiceOffer> offers = offerHash.offersFor(QStringLiteral("image/jpeg"));
         qStableSort(offers); // like kbuildservicefactory.cpp does
-        const QStringList expectedJpegApps = preferredApps["image/jpeg"];
+        const QStringList expectedJpegApps = preferredApps[QStringLiteral("image/jpeg")];
         QCOMPARE(assembleOffers(offers), expectedJpegApps);
 
-        offers = offerHash.offersFor("text/html");
+        offers = offerHash.offersFor(QStringLiteral("text/html"));
         qStableSort(offers);
-        QStringList textHtmlApps = preferredApps["text/html"];
-        if (KService::serviceByStorageId("firefox.desktop")) {
-            textHtmlApps.append("firefox.desktop");
+        QStringList textHtmlApps = preferredApps[QStringLiteral("text/html")];
+        if (KService::serviceByStorageId(QStringLiteral("firefox.desktop"))) {
+            textHtmlApps.append(QStringLiteral("firefox.desktop"));
         }
         qDebug() << assembleOffers(offers);
         QCOMPARE(assembleOffers(offers), textHtmlApps);
 
-        offers = offerHash.offersFor("image/png");
+        offers = offerHash.offersFor(QStringLiteral("image/png"));
         qStableSort(offers);
-        QCOMPARE(assembleOffers(offers), QStringList() << "fakejpegapplication.desktop");
+        QCOMPARE(assembleOffers(offers), QStringList() << QStringLiteral("fakejpegapplication.desktop"));
     }
 
     void testSetupRealFile()
@@ -313,13 +329,13 @@ private Q_SLOTS:
         writeToMimeApps(m_mimeAppsFileContents);
 
         // Test a trader query
-        KService::List offers = KMimeTypeTrader::self()->query("image/jpeg");
+        KService::List offers = KMimeTypeTrader::self()->query(QStringLiteral("image/jpeg"));
         //qDebug() << m_mimeAppsFileContents;
         //qDebug() << "preferred apps for jpeg: " << preferredApps.value("image/jpeg");
         //for (int i = 0; i < offers.count(); ++i) {
         //    qDebug() << "offers for" << "image/jpeg" << ":" << i << offers[i]->storageId();
         //}
-        QCOMPARE(offers.first()->storageId(), QString("fakejpegapplication.desktop"));
+        QCOMPARE(offers.first()->storageId(), QStringLiteral("fakejpegapplication.desktop"));
 
         // Now the generic variant of the above test:
         // for each mimetype, check that the preferred apps are as specified
@@ -346,7 +362,7 @@ private Q_SLOTS:
     void testMultipleInheritance()
     {
         // application/x-shellscript inherits from both text/plain and application/x-executable
-        KService::List offers = KMimeTypeTrader::self()->query("application/x-shellscript");
+        KService::List offers = KMimeTypeTrader::self()->query(QStringLiteral("application/x-shellscript"));
         QVERIFY(offerListHasService(offers, fakeTextApplication, true));
     }
 
@@ -355,23 +371,32 @@ private Q_SLOTS:
         // I removed kate from text/plain, and it would still appear in text/x-java.
 
         // First, let's check our fake app is associated with text/plain
-        KService::List offers = KMimeTypeTrader::self()->query("text/plain");
+        KService::List offers = KMimeTypeTrader::self()->query(QStringLiteral("text/plain"));
         QVERIFY(offerListHasService(offers, fakeTextApplication, true));
 
         writeToMimeApps(QByteArray("[Removed Associations]\n"
                                    "text/plain=faketextapplication.desktop;\n"));
 
-        offers = KMimeTypeTrader::self()->query("text/plain");
+        offers = KMimeTypeTrader::self()->query(QStringLiteral("text/plain"));
         QVERIFY(!offerListHasService(offers, fakeTextApplication, false));
 
-        offers = KMimeTypeTrader::self()->query("text/x-java");
+        offers = KMimeTypeTrader::self()->query(QStringLiteral("text/x-java"));
         QVERIFY(!offerListHasService(offers, fakeTextApplication, false));
     }
 
     void testRemovedImplicitAssociation() // remove (implicit) assoc from derived mimetype
     {
         // #164584: Removing ark from opendocument.text didn't work
-        const QString opendocument = "application/vnd.oasis.opendocument.text";
+        const QString opendocument = QStringLiteral("application/vnd.oasis.opendocument.text");
+
+        // [sanity checking of s-m-i installation]
+        QMimeType mime = QMimeDatabase().mimeTypeForName(opendocument);
+        QVERIFY(mime.isValid());
+        if (!mime.inherits(QStringLiteral("application/zip"))) {
+            // CentOS patches out the application/zip inheritance from application/vnd.oasis.opendocument.text!! Grmbl.
+            QSKIP("Broken distro where application/vnd.oasis.opendocument.text doesn't inherit from application/zip");
+        }
+
         KService::List offers = KMimeTypeTrader::self()->query(opendocument);
         QVERIFY(offerListHasService(offers, fakeArkApplication, true));
 
@@ -381,7 +406,7 @@ private Q_SLOTS:
         offers = KMimeTypeTrader::self()->query(opendocument);
         QVERIFY(!offerListHasService(offers, fakeArkApplication, false));
 
-        offers = KMimeTypeTrader::self()->query("application/zip");
+        offers = KMimeTypeTrader::self()->query(QStringLiteral("application/zip"));
         QVERIFY(offerListHasService(offers, fakeArkApplication, true));
     }
 
@@ -389,7 +414,7 @@ private Q_SLOTS:
     {
         // #178560: Removing ark from interface/x-winamp-skin didn't work
         // Using application/x-kns (another zip-derived mimetype) nowadays.
-        const QString mime = "application/x-kns";
+        const QString mime = QStringLiteral("application/x-kns");
         KService::List offers = KMimeTypeTrader::self()->query(mime);
         QVERIFY(offerListHasService(offers, fakeArkApplication, true));
 
@@ -399,7 +424,7 @@ private Q_SLOTS:
         offers = KMimeTypeTrader::self()->query(mime);
         QVERIFY(!offerListHasService(offers, fakeArkApplication, false));
 
-        offers = KMimeTypeTrader::self()->query("application/zip");
+        offers = KMimeTypeTrader::self()->query(QStringLiteral("application/zip"));
         QVERIFY(offerListHasService(offers, fakeArkApplication, true));
     }
 
@@ -414,13 +439,13 @@ private Q_SLOTS:
         writeToMimeApps(QByteArray("[Removed Associations]\n"
                                    "application/x-desktop=faketextapplication.desktop;\n"));
 
-        offers = KMimeTypeTrader::self()->query("text/plain");
+        offers = KMimeTypeTrader::self()->query(QStringLiteral("text/plain"));
         QVERIFY(offerListHasService(offers, fakeTextApplication, true));
 
-        offers = KMimeTypeTrader::self()->query("application/x-desktop");
+        offers = KMimeTypeTrader::self()->query(QStringLiteral("application/x-desktop"));
         QVERIFY(!offerListHasService(offers, fakeTextApplication, false));
 
-        offers = KMimeTypeTrader::self()->query("application/x-theme");
+        offers = KMimeTypeTrader::self()->query(QStringLiteral("application/x-theme"));
         QVERIFY(!offerListHasService(offers, fakeTextApplication, false));
     }
 
@@ -429,18 +454,14 @@ private:
 
     void runKBuildSycoca()
     {
-        //qDebug();
         // Wait for notifyDatabaseChanged DBus signal
         // (The real KCM code simply does the refresh in a slot, asynchronously)
-        QEventLoop loop;
-        QObject::connect(KSycoca::self(), SIGNAL(databaseChanged(QStringList)), &loop, SLOT(quit()));
-        QProcess proc;
-        const QString kbuildsycoca = QStandardPaths::findExecutable(KBUILDSYCOCA_EXENAME);
-        QVERIFY(!kbuildsycoca.isEmpty());
-        proc.setProcessChannelMode(QProcess::MergedChannels); // silence kbuildsycoca output
-        proc.start(kbuildsycoca, QStringList() << "--testmode");
-        proc.waitForFinished();
-        loop.exec();
+        QSignalSpy spy(KSycoca::self(), SIGNAL(databaseChanged(QStringList)));
+        KBuildSycoca builder;
+        QVERIFY(builder.recreate());
+        if (spy.isEmpty()) {
+            spy.wait();
+        }
     }
 
     void writeToMimeApps(const QByteArray &contents)
